#!/usr/bin/python
#
# config-test.py - python routine to test different config options
#
# To Do for config-test.py
#
# To Do for config options:
#
# To Do for test framework:
#
# BUGS:
#
# CHANGELOG:
# 0.1.0 - Copied from config-sizes-test.py, version 0.8.1

import os, sys
import commands
import re
import time
import random
import ttc_test_utils

MAJOR_VERSION = 0
MINOR_VERSION = 1
REVISION = 0

######################################
# Define some globals for this test suite

test_suite_name="Config-test"
src_dir = "test-linux"
test_data_dir_top = "../test-data"
default_wiki_path = "oak:/usr/share/moin/alpwiki/data/text/ConfigSizeTestResults"

yes_no_options = [
	"BASE_FULL", "NET_SMALL", "CC_FUNIT_AT_A_TIME",
	"LINUXTINY_DO_UNINLINE",
	"PANIC", "FULL_PANIC",
	"SYSFS_DEPRECATED",
	"SYSFS",
	"ETHTOOL", "INETPEER", "NET_SK_FILTER", "NET_DEV_MULTICAST",
	"IGMP", "IP_MULTICAST", "BINFMT_SCRIPT", "BINFMT_ELF_AOUT",
	"PRINTK", "BUG", "ELF_CORE", "PROC_KCORE", "SYSENTER", "AIO", "XATTR",
	"FILE_LOCKING", "DIRECTIO", "KALLSYMS", "KALLSYMS_ALL",
	"SHMEM", "SWAP", "SYSVIPC",
	"SYSVIPC_SYSCTL", "PROC_SYSCTL", "SYSCTL_SYSCALL",
	"PROC_FS",
	"UID16",
	"MODULES", "ISA", "KMOD",
	"DAB", "VGA_CONSOLE",
	"USB", "HID", "PCI", "XIP_KERNEL",
	"BLK_DEV_LOOP", "BLK_DEV_RAM",
	"IOSCHED_AS", "IOSCHED_DEADLINE", "IOSCHED_CFQ",
	"IP_PNP", "IP_PNP_DHCP",
	"IDE", "SCSI",
	"EXT2_FS", "EXT3_FS", "INOTIFY", "AUTOFS_FS", "AUTOFS4_FS", "NFS_FS",
	"DLM", "PROFILING", "KPROBES",
	"ENABLE_MUST_CHECK",
	"UNUSED_SYMBOLS", "DEBUG_KERNEL", "DOUBLEFAULT",
	"CRYPTO", "SERIAL_PCI", "PCI_QUIRKS",
	"CRC32_TABLES",
	"MEMPOOL", "CONSOLE_TRANSLATIONS",
]

option_list = [("MAX_SWAPFILES_SHIFT", ('5','1','0')),
	("NR_LDISCS", ('16','1')),
	("MAX_USER_RT_PRIO", ('100', '5')),
	# 12 is the smallest LOG_BUF_SHIFT allowed
	("LOG_BUF_SHIFT", ('17', '14', '12')),
	("IDE_HWIFS", ('0', '16', '1')),
	#("SLAB",('n',('y',[("SLUB",'n'),("SLOB",'n')]) )),
	("SLUB",('n',('y',[("SLAB",'n'),("SLOB",'n'),("SLUB_DEBUG",'n')]) )),
	("SLOB",('n',('y',[("SLAB",'n'),("SLUB",'n')]) )),
	("SYSCTL",('y',('n',[("SYSVIPC_SYSCTL",'n'),("PROC_SYSCTL",'n'),
		("SYSCTL_SYSCALL",'n')]) )),
]

test_list = [
	("SYSCTL",('y',('n',[("SYSVIPC_SYSCTL",'n'),("PROC_SYSCTL",'n'),
		("SYSCTL_SYSCALL",'n')]) )),
]

######################################
# Define some convenience classes and functions

def usage():
	print """Usage: %s <options> <target>

where <options> are shown below, and <target> is the name of the
target for the 'ttc' command,

By default, this program builds a baseline kernel, and several kernels
in alternative configurations.

 -h     Show this usage help
 -c     Continue building from a previous run.  That is, if the output
        files for an option already exist, skip the build for those options.
 -a-	Omit building the configuration alternatives.
        (Only make the baseline kernel)
 -b-    Omit building the baseline configuration.
 -r     Perform runtime test of free memory
 -o <config_exp>  Specify a one-shot build, for the config option specified.
 -t     Build the 'test' options, instead of the default option list.
 -V     Show version information

For a one-shot build, the program will build the baseline kernel,
and then perform alternate builds, altering only one specified
CONFIG variable.  The config expression can be a simple or compound.
Some examples are:
 -o AIO=y|n
 -o AIO
 -o LOG_BUF_SHIFT=9|11|14|17

An equal sign separates the option name from the list of values to try.
Values are separated from each other by vertical bars.  If the equal
sign and values are omitted, then the values "y" and "n" are used.
The first two examples above are equivalent.
""" % os.path.basename(sys.argv[0])

def test_setup(test_run):
	global build_dir
	# prepare for a test run
	# 1. get kernel
	# 1.1 make build dir
	# 1.2 make test data dir
	# 2. get default config

	print "Doing test preparation for %s tests..." % test_run.suite_name

	# auto-detect that kernel is already present in src_dir
	# and skip the get_kernel automatically
	rcode = test_run.do_command('test -f %s/gk_marker' % src_dir)
	if rcode:
		test_run.do_ttc("get_kernel -o %s" % src_dir, 1)
		test_run.do_command('echo "get_kernel completed" >%s/gk_marker' % src_dir)
	else:
		print "*** Skipping get_kernel (get_kernel marker found)"

	os.chdir(src_dir)
	build_dir = "../test-build/%s" % test_run.target
	os.environ["KBUILD_OUTPUT"]=build_dir
	test_run.do_command("install -d %s" % build_dir, 1)
	test_run.do_command("install -d %s" % test_data_dir, 1)

	# create baseline config, and save it
	test_run.do_ttc("get_config", 1)
	test_run.do_ttc("set_config CONFIG_EMBEDDED=y")
	test_run.do_make_oldconfig()
	test_run.do_command("cp %s/.config %s/config.baseline" % \
			(build_dir, test_data_dir))

def test_baseline(test_run):
	global runtime_test

	test_run.set_id("baseline", "baseline kernel configuration")
	print "Running test %s..." % test_run.id

	build_dir = os.environ["KBUILD_OUTPUT"]

	# get baseline config
	test_run.do_command("cp %s/config.baseline %s/.config" % \
			(test_data_dir, build_dir))

	test_run.do_make_oldconfig()

	# remove last kernel built, if any
	rcode = test_run.do_command("rm %s/vmlinux" % build_dir, 0, 1)

	# clear localversion for better ccache-ing
	test_run.clear_localversion()

	# build kernel
	rcode = test_run.do_ttc("kbuild")
	if rcode or not os.path.exists("%s/vmlinux" % build_dir):
		test_run.failure("Could not build baseline kernel")
		return

	# get size data for baseline kernel
	save_size_data(test_run, "%s/vmlinux" % build_dir, "baseline", "-")

	# save the kernel and map files into test_data_dir
	test_run.do_command("cp %s/vmlinux %s/vmlinux.baseline" % \
			(build_dir, test_data_dir))
	test_run.do_command("cp %s/System.map %s/System.map.baseline" % \
			(build_dir, test_data_dir))

	if runtime_test:
		do_runtime_test(test_run)
	else:
		print "  No runtime test requested."

def save_size_data(test_run, kernel_file, config_name, value):
	rcode, result = test_run.do_command_result("size %s" % kernel_file)
	if rcode:
		test_run.failure("Could not get size for kernel")
	else:
		rline = result.split('\n')[1]
		text_size = rline.split()[0]
		data_size = rline.split()[1]
		bss_size = rline.split()[2]
		tot_size = rline.split()[3]
		test_run.result_out("kernel text size: %s" % text_size)
		test_run.result_out("kernel data size: %s" % data_size)
		test_run.result_out("kernel bss size: %s" % bss_size)
		test_run.result_out("kernel total size: %s" % tot_size)

def get_free_mem(test_run):
	(rcode, result) = test_run.run_on_target("free -tb")
	free_mem = "error parsing result='%s'" % result
	if rcode:
		test_run.failure("Error collecting results from free")
		return 0

	try:
		lines = result.split("\n")
		for line in lines:
			if line.startswith("Total:"):
				free_mem = line.split()[3]
	except:
		pass

	if free_mem.find("error") == -1:
		test_run.result_out("free memory: %s" % free_mem, result )
		return free_mem
	else:
		test_run.failure("Error parsing results from free", result)
		return 0

def do_runtime_test(test_run):
	# set localversion for runtime testing
	test_run.set_localversion()

	# build kernel
	rcode = test_run.do_ttc("kbuild")
	if rcode or not os.path.exists("%s/vmlinux" % build_dir):
		test_run.failure("Could not build kernel with localversion")
		return

	# install kernel
	rcode = test_run.do_ttc("kinstall", 1)
	if rcode:
		test_run.failure("Could not install kernel")
		return

	# reset target
	rcode = test_run.reset_target("reset")

	# verify that the running kernel is the one just built
	kernel_ok = test_run.check_localversion()
	if not kernel_ok:
		# try a hard reboot, in case reset didn't work
		test_run.reset_target("reboot")
		kernel_ok = test_run.check_localversion()
		if not kernel_ok:
			test_run.failure("Kernel just built is not running on board")
			return

	# get used mem on machine
	get_free_mem(test_run)


# test an individual option setting 
def test_one_option(test_run, option, value, skip_already_built):
	global runtime_test

	# handle config dependencies here
	deps = None
	if type(value) != type("y"):
		# we have a tuple, break out the option value and the 
		# dependencies
		deps = value[1]
		value = value[0]

	config_name = option[7:]
	test_id="%s=%s" % (config_name, value)
	test_run.set_id(test_id, "option test")
	print "Running test %s..." % test_run.id

	target = test_run.target
	build_dir = os.environ["KBUILD_OUTPUT"]

	if skip_already_built:
		# check for an already-built option
		opfilename="vmlinux.%s-%s" % (config_name, value)
		if os.path.exists("%s/%s" % (test_data_dir, opfilename)):
			print "%s already exists, skipping..." % opfilename
			return

	# set specific configuration value
	test_run.do_command("cp %s/config.baseline %s/.config" % \
			(test_data_dir, build_dir))

	rcode = test_run.do_command("ttc %s set_config %s=%s" % (target, option, value))
	if deps:
		for do, dv in deps:
			rcode = test_run.do_command("ttc %s set_config CONFIG_%s=%s" % (target, do, dv))

	test_run.do_make_oldconfig()

	# save off .config for test script debugging
	rcode = test_run.do_command("cp %s/.config %s/config.%s-%s" % (build_dir, test_data_dir, config_name, value))
	
	# verify this config was accepted
	if value=='n':
		option_line = "%s is not set" % option
	else:
		option_line = "%s=%s" % (option, value)
	rcode = test_run.do_command('grep "%s" %s/.config' % (option_line, build_dir))
	#print "grep rcode=%d" % rcode
	if rcode:
		test_run.failure("Could not set configure option %s to value %s" % (option, value))
		return

	cmd = "diff %s/config.baseline %s/config.%s-%s | grep %s >/dev/null" % \
		(test_data_dir, test_data_dir, config_name, value, option)

	print "  Executing '%s'" % cmd
	rcode = os.system(cmd)
	if rcode:
		# no diff found, option is same as in baseline, skip it
		test_run.result_out(
			'Setting matches baseline - skipping build')
		return

	# try diffconfig, if it works, save the result
	cmd = "diffconfig %s/config.baseline %s/config.%s-%s" % \
		(test_data_dir, test_data_dir, config_name, value)
	rcode, result = test_run.do_command_result(cmd)
	if not rcode:
		test_run.result_out("diffconfig result: %s" % result)

	# remove last kernel built, if any
	(rcode) = test_run.do_command("rm %s/vmlinux" % build_dir, 0, 1)

	# clear localversion for (possibly?) better ccache-ing
	test_run.clear_localversion()

	# build kernel
	rcode = test_run.do_ttc("kbuild")
	if rcode or not os.path.exists("%s/vmlinux" % build_dir):
		test_run.failure("Could not build kernel")
		return

	# save off kernel and map
	rcode = test_run.do_command("cp %s/vmlinux %s/vmlinux.%s-%s" % \
		(build_dir, test_data_dir, config_name, value))
	rcode = test_run.do_command("cp %s/System.map %s/System.map.%s-%s" % \
		(build_dir, test_data_dir, config_name, value))

	# get size data for kernel
	save_size_data(test_run, "%s/vmlinux" % build_dir, config_name, value)

	if runtime_test:
		do_runtime_test(test_run)
	else:
		print "  No runtime test requested."


def main():
	global option_list, build_dir, test_data_dir
	global runtime_test

	do_baseline = 1
	do_options = 1
	do_test_options = 0
	do_one_shot = 0
	skip_already_built = 0
	runtime_test = 0
	do_wiki_write = 0

	if '-h' in sys.argv:
		usage()
		sys.exit(1)

	if '-V' in sys.argv:
		print "config-test.py Version %d.%d.%d" % \
			(MAJOR_VERSION, MINOR_VERSION, REVISION)
		sys.exit(1)

	if '-a-' in sys.argv:
		do_options = 0
		print "Not building configuration alternatives."
		sys.argv.remove("-a-")

	if '-c' in sys.argv:
		skip_already_built = 1
		print "Not building options already built." 
		sys.argv.remove("-c")

	if '-b-' in sys.argv:
		do_baseline = 0 
		print "Not building baseline kernel."
		sys.argv.remove("-b-")

	if '-o' in sys.argv:
		do_one_shot = 1
		one_shot_config = sys.argv[sys.argv.index("-o")+1]
		sys.argv.remove("-o")
		sys.argv.remove(one_shot_config)
		print "Performing one-shot build with config option %s" % \
			one_shot_config

	if '-r' in sys.argv:
		runtime_test = 1
		print "Performing runtime test of free memory"
		sys.argv.remove("-r")

	if '-t' in sys.argv:
		do_test_options = 1
		print "Building the test options, instead of the regular options." 
		sys.argv.remove("-t")


	# check for a remaining arg
	if len(sys.argv)<2:
		print "Error: missing target to run test on."
		usage()
		sys.exit(1)
	else:
		target = sys.argv[1]

	# verify that target is supported by 'ttc'
	(rcode, result) = commands.getstatusoutput('ttc list -q')
	if rcode:
		print "Error: Problem running 'ttc list'"
		sys.exit(1)

	tlist = result.split("\n");
	if target not in tlist:
		print "Error: target '%s' not supported on this host." % target
		print "Available targets are:"
		for t in tlist:
			print "   %s" % t
		print
		usage()
		sys.exit(1)

	test_run = ttc_test_utils.test_run_class(test_suite_name, target)
	test_data_dir = test_data_dir_top + "-" + target

	# make directory for test output
	# (remove leading ../ from test_data_dir, since we haven't
	# changed to the kernel source dir yet.)
	test_run.do_command("install -d %s" % test_data_dir[3:], 1)

	# log results
	test_run.start_log(test_data_dir[3:])

	# Here is the actual testing
	print "Running tests on target: %s" % target

	test_setup(test_run)

	if do_baseline:
		test_baseline(test_run)

	if do_one_shot:
		# parse one_shot_config
		if one_shot_config.find('=')==-1:
			one_shot_option = one_shot_config
			values = ('y|n')
		else:
			(one_shot_option, values) = one_shot_config.split('=')
		one_shot_values = values.split('|')

		option_list = [(one_shot_option, one_shot_values)]

		# override silly people who specify an option to test, but
		# turn off do_options.
		do_options = 1
	else:
		# add yes_no options to default options list
		# put yes/no options at front of list
		for option in yes_no_options:
			option_list.insert(0, (option, ('y','n')))

	# now test each option
	if do_options:
		# set to the alternate list, if requested
		if do_test_options:
			option_list = test_list

		total_options = len(option_list)
		cur_option = 1
		for (option, values) in option_list:
			print "Now on option %d, out of %d (time=%s)" % \
				(cur_option, total_options, time.localtime())
			option = "CONFIG_"+option
			for value in values:
				test_one_option(test_run, option, value,
					skip_already_built)
			cur_option += 1

	# close the log results
	test_run.close_log()

	# show a results summary
	print "\n###########################################"
	print "Results summary:"
	test_run.show_summary_results()

main()
