#!/usr/bin/python
#
# do-build.py - a python script to perform a full ALP build

# here are some customization variables
do_checkout = 1		# perform checkout of build-distro area
make_work_dirs = 1
remove_work_area = 1	# remove work area before starting
get_package_sources = 1
make_kernel_archives = 1

# specify which architectures to buld for
arch_list = ["i386", "mips", "powerpc", "arm"]
#arch_list = ["arm"]

#
# An ALP build consists of the following parts:
#  * clean any junk from previous runs
#  + do a pristine distro checkout
#  + get alp-package sources
#  + make kernel archives
#  + build the ALP packages
#  * make the release notes
#  + copy distro files to oak
#  * install headers locally (every night??) 
# 
# To Do:
#  * clean any junk from previous runs
#  * install headers locally (every night??) 
#  * make the release notes
# 
# Bugs:
# ARM sony-target-dev-alp-linux-tests doesn't build 
#  it fails making kprobe-tests, because of missing
#  /devel/bin/../target/lib/libpthread.so.0

import os, sys
import commands
import time

MAJOR_VERSION=0
MINOR_VERSION=5
REVISION=3

distro_dir = "build-distro"
work_dir = "work-2.4"

# this must match what is in sony-cross-kbuild.spec and
# sony-cross-kernel-headers.spec
release_id = "2.4"

last_time = time.time()
start_time = time.strftime("%Y-%m-%d_%H:%M:%S")

# FIXTHIS - don't hardcode work directory in log path
log_file = "/home/tbird/work/nightly-build/logs/log--%s" % start_time

# parse -v on command line
verbose = 0
if '-v' in sys.argv:
	verbose = 1

def usage():
 	print """Usage: do-build.py [-h][-v][-V]

Edit do-build itself to modify the list of architectures to build for,
and to set flags for whether to do a build-distro checkout, to remove
the work directory, etc.

Options:
-h, --help     Show this usage help
-v             Show verbose messages
-V, --version  Show version and exit
"""

# FIXTHIS - make sure everything builds from completely clean sources
# remove any existing build-distro directory

def log_msg(msg):
	global log_file
	fd = open(log_file, "a")
	fd.write(msg+'\n')
	fd.close()

def show_msg(msg):
	global last_time
	cur_time = time.time()
	delta = cur_time - last_time
	delta_min = int(delta/60)
	delta_sec = int(delta-delta_min*60)
	last_time = cur_time
	time_str = time.strftime("%m-%d %H:%M:%S") + \
		" (%d:%02d m)" % (delta_min, delta_sec)
	out_str = "[%s] Executing: %s" % (time_str, msg)

	# print and log the message
	print out_str

	log_msg("="*70)
	log_msg(out_str)

def do_command(cmd, msg, die_on_error=0):
	global verbose

	show_msg(msg)
	(rcode, result) = commands.getstatusoutput(cmd)
	if rcode:
		print "Error %d running command to %s" % (rcode, msg)
		print "Result='%s'" % result
		if die_on_error:
			sys.exit(1)
	out_str = "Result='%s'" % result
	log_msg(out_str)
	if verbose:
		print out_str
	return rcode

def main():
	global start_time
	global do_checkout

	if "-V" in sys.argv or "--version" in sys.argv:
		print "do-build.py version %d.%d.%d" % (MAJOR_VERSION, MINOR_VERSION, REVISION)
		sys.exit(0)

	if "-h" in sys.argv or "--help" in sys.argv:
		usage()
		sys.exit(0)

	# put main banner in log
	log_msg("="*70)
	banner = "Starting ALP build at %s" % start_time
	log_msg(banner)
	log_msg("="*70)

	if do_checkout:
		# check out the build distro
		cmd = "tla get alp@oak--linux-3/alp-process--build-distro--2.0 %s" % distro_dir
		do_command(cmd, "Check out build-distro", 1)
	else:
		show_msg("Skipping check out of build-distro")

	if make_work_dirs:
		show_msg("Making work directories")

		# change to the top of the distro dir and make work directory
		os.chdir(distro_dir)

		# remove old work area if present
		if remove_work_area:
			cmd = "rm -rf %s" % work_dir
			do_command(cmd, "Remove old work area (%s)" % work_dir, 1)

		os.mkdir(work_dir)
	

	if get_package_sources:
		# get alp-package sources
		cmd = "./alp-linux-2.0/get-package-sources -v -w %s" % work_dir
		do_command(cmd, "Get ALP package sources", 1)

	if make_kernel_archives:
		# make kernel archives
		cmd = "./make-kernel-archives -v -w %s -i %s alp-linux--dev-2" % (work_dir, release_id)
		do_command(cmd, "Make kernel archives", 1)

	# FIXTHIS -  should build and install the kernel header packages first,
	# then build all other packages using these headers.

	# build ALP packages, for each arch
	# see arch_list at top of file
	for arch in arch_list:
		cmd = "./build-alp-distros ./alp-linux-2.0 %s %s" % (work_dir, arch)
		do_command(cmd, "Build ALP packages for %s" % arch)


	show_msg("Done.")

main()

