#!/bin/sh
#
# This command automatically creates an LTSI kernel tree by combining
# the ltsi-kernel patch queue with the linux-stable tree
# the results are left in a directory called 'linux-stable'

# stop if already in a kernel source tree
if [ -e MAINTAINERS ] ; then
	echo "Already in a kernel source tree; aborting..."
	exit 1;
fi

# clone or pull the ltsi-kernel patch queue
echo "Retrieving LTSI patch queue..."
if [ -d ltsi-kernel ] ; then
	cd ltsi-kernel ; git pull ; cd ..
else
	git clone http://git.linuxfoundation.org/ltsi-kernel.git
fi
export KERNEL_VERSION=$(cat ltsi-kernel/KERNEL_VERSION)
echo Kernel version=$KERNEL_VERSION

# clone or pull the linux-stable tree
echo "Retrieving linux-stable tree"
if [ -d linux-stable ] ; then 
	cd linux-stable ; git pull ; cd ..
else
	git clone http://git.kernel.org/pub/scm/linux/kernel/git/stable/linux-stable.git
fi

# apply LTSI patches to stable 
echo "Applying LTSI patches to stable"
cd linux-stable
git checkout -b ltsi-patches v$KERNEL_VERSION  

# if there are patches already applied, git rid of them
quilt pop -a

# link to the patches directory
ln -s ../ltsi-kernel patches
# apply all the patches
quilt push -a
