#!/usr/bin/python

import sys
import matplotlib.pyplot as plt
import numpy as np
import fileinput

test_data = [2,4,6,3,5,12,4,7,6,9,10,5,4,5,5,6,4,3,3,\
	4,3,3,3,5,6,7,7,8,1,2,2,2,1,1,1,1,1,1,2,3,3,1]

def usage():
    print """Usage: histogram.py [options] [<files>]

Draw a histogram with lines, from data provided in the indicated files.
If no files are specified, data is read from standard input.

Options:
 -h, --help   show this usage help
 -b <bins>    specify the number of bins for the histogram
 -t <title>   specify the title for the chart
 --test       use test data (internal to program) instead of reading
              the data from inputs
"""
    sys.exit(0)

def get_option_value(option):
    index = sys.argv.index(option)
    try:
        value = sys.argv[index+1]
    except:
        print "Error: Missing value for option '%s'" % option
        usage()

    del(sys.argv[index])
    del(sys.argv[index])
    return value

def main():
    num_bins = 100
    title = ""

    # parse arguments
    if '-h' in sys.argv or '--help' in sys.argv:
	usage()
    if '-b' in sys.argv:
	num_bins = int(get_option_value('-b'))
    if '-t' in sys.argv:
        title = get_option_value('-t')
    if "--test" in sys.argv:
	data = test_data
    else:
	data = []
	for line in fileinput.input():
		value = float(line)
		data.append(value)

    #print "data=", data

    # create the histogram
    bins, edges, patches = plt.hist(data, num_bins, color="red") 
    #print "bins=", bins
    #print "edges=", edges
    #print "patches=", patches 

    xlo = min(data)
    xhi = max(data)
    mode = np.bincount(data).argmax()
    ylo = 0
    yhi = max(bins)

    print "min=", xlo, "max=", xhi, "mode=", mode

    plt.title(title)
    plt.axis([xlo, xhi, ylo, yhi])
    plt.show()

if __name__ == "__main__":
	main()
