#!/bin/sh
#
# update-recovery-image.sh - a program to update the kernel
# in the recovery partition of an android phone, with one specified
#
# Written by: Tim Bird <tim.bird (at) am.sony.com>
# Copyright 2009 Sony Corporation
#

function usage {
	echo "Usage: update-recovery-image [<new-zImage>]"
	echo "where <new-zImage> is the path to the new kernel image to install"
	echo "in the recovery partition of the device."
}

# make sure image manipulation tools are available

unpack=$(which unpack-bootimg.pl 2>/dev/null)
if [ -z "$unpack" ] ; then 
	echo "Error: missing unpack-bootimg.pl"
	exit 1
fi

repack=$(which repack-bootimg.pl 2>/dev/null)
if [ -z "$repack" ] ; then 
	echo "Error: missing repack-bootimg.pl"
	exit 1
fi

mkbootimg=$(which mkbootimg 2>/dev/null)
if [ -z "mkbootimg" ] ; then 
	echo "Error: missing mkbootimg"
	exit 1
fi

adb=$(which adb 2>/dev/null)
if [ -z "adb" ] ; then 
	echo "Error: missing adb"
	exit 1
fi

if [ -z "$1" ] ; then
	usage
	exit
fi

if [ $1 == "-h" -o "$1" == "--help" ] ; then
	usage
	exit
fi

# retrieve recovery.img into current directory
$adb pull /system/recovery.img .

# extract the contents
$unpack recovery.img

# contents in current directory are now:
#    recovery.img-kernel.gz
#    recovery.img-ramdisk.cpio.gz
#    recovery.img-ramdisk (directory)

# make the "new-recovery.img" image
$repack $1 recovery.img-ramdisk new-recovery.img

# now put it on the phone
adb shell mount -o rw,remount -t yaffs /dev/block/mtdblock3 /system
adb push new-recovery.img /system/recovery.img
adb shell flash_image recovery /system/recovery.img

echo
echo "Now reboot the phone, holding down the 'home' button on boot"
echo "to boot into the new kernel."
echo 
echo "To restore the previous recovery image, do the following:"
echo " * do a normal phone boot (not in recovery mode)"
echo " * $ adb shell mount -o rw,remount -t yaffs /dev/block/mtdblock3 /system"
echo " * $ adb push recovery.img /system/recovery.img"
echo " * $ adb shell flash_image recovery /system/recovery.img"
echo " * reboot in recovery mode"


# clean up files
rm recovery.img-kernel.gz recovery.img-ramdisk.cpio.gz
rm -rf recovery.img-ramdisk

# leave recovery.img, in case user
